// AUTO-GENERATED BY UpdateGenVersion.bat 1895fbfbde86dc0e45e0f1a286ee44c3c001fb7f
const char GenVersion_Version[] = "0.4.4-dev";
const char GenVersion_Commit[] = "1895fbfbde86dc0e45e0f1a286ee44c3c001fb7f";
